function X = dsp_fft(x,N)
% X = dsp_fft(x,N)
% Compute X fft coeficients based on discrete input x
% If N is not power of two, increase N to power of 2
% Then pad x with zeros or truncate as N determines
% inputs:
% x -> input data vector
% N -> length of fft
% outputs:
% X -> fft of x, length(X) = N
% dsp_fft.m
% author: Nicholas Kottenstette
% date:   11/9/2004
% performance:
% N = 2^16;
% x = sin(0.25*pi*[0:N-1]);
% baseline to compare:
% tic;fft(x,N);toc     -> 0.14 seconds
% tic;dsp_fft(x,N);toc -> 11.5 seconds
  [m,n] = size(x);
  % Check x is a vecor and will keep as a single column vector
  if (m > 1) & (n > 1)
    error('dsp_fft:invalid_input','x not a vector m = %d, n = %d\n"',m,n);
  elseif m < n
    x = x';
    [m,n] = size(x);
  end
  if N < 2
    N = m;
  end
  N = bitshift(1,ceil(log2(N)));
  Nx = m;
  % zero pad or truncate x
  if Nx < N
    x = [x;zeros(N-Nx,1)];
  elseif Nx > N
    x = x(1:N,1);
  end
  % Shuffle inputs
  [xs,Smax] = shuffle(x,N);
  % Calculate fft using sparse matrices
  X = mfft(xs,Smax);
  
function [xs,Smax] = shuffle(x,N)
  Smax = log2(N);
  ind = [0:2^Smax-1]';
  rind = bitrevorder(ind);
  xs = x(rind+1);
  
function X = mfft(x,Smax)
  fp = fopen('fft_coef/coef.mat');
  if fp == -1
    c = gen_coef();
  else
    fclose(fp);
    fp = load('fft_coef/coef.mat');
    c = fp.c;
  end
  X = x;
  N = 2^Smax;
  Xt = zeros(size(X));
  for s = 1:Smax
    %F = spalloc(N,N,2*N);
    n = bitshift(1,s);
    cis = bitshift(1,s-1);
    cie = bitshift(1,s)-1;
    %f_ul = speye(n/2);
    %f_ur = sparse(diag(c(cis:cie)));
    f_un = n/2;
    fn = n;
    %f = [f_ul f_ur;f_ul -f_ur];
    %[fn,fn]=size(f);
    co = c(cis:cie);
    imax = bitshift(1,Smax-s);
    Xt = 0*Xt;
    for i = 1:imax
      fns1 = fn*(i-1) + 1;
      fne1 = fns1 + f_un - 1;
      fns2 = fne1 + 1;
      fne2 = fns2 + f_un - 1;
      Xt(fns1:fne1,1) = X(fns1:fne1,1) + co.*X(fns2:fne2,1);
      Xt(fns2:fne2,1) = X(fns1:fne1,1) - co.*X(fns2:fne2,1);
    end
    X = Xt;
  end
  
function c = gen_coef()
  SMAX = 16;
  NcMax = 2^SMAX;
  ind = 1;
  for s = 1:SMAX
    L = 2^s;
    for i = 0:2^(s-1)-1
      c(ind,1) = exp(-j*2*pi*i/L);
      ind = ind + 1;
    end
  end
  save fft_coef/coef.mat c
  